local items = {}

items = {
	{text = "Back",onclick = function(self) menu.pop() end},
	{
		text = "Style",
		init = function(self)
			self.options = love.filesystem.getDirectoryItems("/games/" .. settings.game .. "/style")
			self.selected = getindex(self.options, settings.style)
		end,
		is_highlighted = function(self)
			return self.options[self.selected] ~= settings.style
		end,
		onclick = function(self)
			local s = self.options[self.selected]

			menu.push({{
				text = "This may reset your game! Are you sure?",
				onclick = function(self)
					if self.selected == 1 then
						menu.pop()
						menu.push({{
							text = "Set this style's defaults?",
							onclick = function(self)
								settings.style = s
								newgame("style")
								if self.selected == 1 then
									settings.conf = shallowcopy(style.conf)
									game.reload.gfx()
									game.reload.sfx()
									game.reload.music()
								end
								menu.pop()
								menu.init()
							end,
							selected = 1,
							options = {"Yes", "No"},
							type = "newline",
						}})

					else
						menu.pop()
					end
				end,
				selected = 1,
				options = {"Yes", "No"},
				type = "newline",
			}})

		end,
		selected = 1,
		options = {},
		help = "Select to apply",
	},
	{
		text = "Style-Specific Options",
		init = function(self)
			if exists("/games/minobueno/style/" .. settings.style .. "/menu.lua") then
				self.table = love.filesystem.load("/games/minobueno/style/" .. settings.style .. "/menu.lua")()
				self.table.tabname = "Style"
				-- init all menu items
				for i,k in pairs(self.table) do if k.init then k.init(k) end end
			else
				self.table = {{
					text = "This style has no options",
					onclick = function(self) menu.pop() end,
					selected = 1,
					options = {"Ok"},
					type = "newline",
				}}
			end
			table.insert(self.table, 1, {
				text = "Back",
				onclick = function(self) menu.pop() end
			})
		end,
		onclick = function(self)
			menu.push(self.table)
		end,
	},
	{
		text = "SFX",
		init = function(self)
			self.options = {}
			for i,k in pairs(love.filesystem.getDirectoryItems("games/" .. settings.game .. "/sfx/")) do
				self.options[i] = k
			end
			table.insert(self.options, "none")
			self.selected = getindex(self.options, settings.conf.piece_sfx)
		end,
		onchange = function(self)
			settings.conf.piece_sfx = self.options[self.selected]
			game.reload.sfx()
		end,
		selected = 1,
		options = {},
	},
	{
		text = "Piece GFX",
		init = function(self)
			self.options = love.filesystem.getDirectoryItems("games/" .. settings.game .. "/pieces/")
			for i,k in ipairs(self.options) do -- trim file extension
				self.options[i] = k
			end
			self.selected = getindex(self.options, settings.conf.piece_gfx)
			if not self.selected then self.selected = 1 end
		end,
		onchange = function(self)
			settings.conf.piece_gfx = self.options[self.selected]
			game.reload.gfx()--block.tex = lg.newImage("games/" .. settings.game .. "/pieces/" .. settings.conf.piece_gfx .. ".png")
		end,
		selected = 1,
		options = {},
	},
	{
		text = "Ghost Piece",
		init = function(self)
			if settings.conf.ghost then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			if self.selected == 1 then
				settings.conf.ghost = true
			else
				settings.conf.ghost = false
			end
			settings.conf.ghost = self.options[self.selected]
			redraw.game = true
		end,
		selected = 1,
		options = {true, false},
		options_text = {"Shown","Hidden"},
		help = "Shows where your piece will land if you hard drop",
	},
	{
		text = "Spin Candy",
		init = function(self)
			if settings.conf.spinny then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			settings.conf.spinny = self.options[self.selected]
			redraw.game = true
		end,
		selected = 1,
		options = {true, false},
		options_text = {"Enabled","Disabled"},
		help = "Makes the pieces rotate smoothly instead of instantaneously",
	},
	{
		text = "Smooth falling",
		init = function(self)
			self.selected = getindex(self.options, settings.conf.smoothfall)
		end,
		onchange = function(self)
			settings.conf.smoothfall = self.options[self.selected]
			redraw.game = true
		end,
		selected = 1,
		options = {false, "early", "late"},
		options_text = {"Disabled", "Early", "Late"},
		help = "Makes the pieces fall per-pixel instead of per-block",
	},
	{
		text = "Smooth sliding",
		init = function(self)
			self.selected = getindex(self.options, settings.conf.smoothslide)
		end,
		onchange = function(self)
			settings.conf.smoothslide = self.options[self.selected]
			redraw.game = true
		end,
		selected = 0,
		options = {0, 1, 2, 3, 4},
		help = "Frames of smooth sliding, left and right",
	},
	{
		text = "Show Next Piece",
		init = function(self)
			if settings.conf.next then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			settings.conf.next = self.options[self.selected]
			redraw.game = true
		end,
		selected = 1,
		options = {true, false},
		options_text = {"Shown","Hidden"},
		help = "Shows/hides the next piece. Some games map this to its own button.",
	},
	{
		text = "Hold Piece",
		init = function(self)
			if settings.conf.hold then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			settings.conf.hold = self.options[self.selected]
			redraw.game = true
		end,
		selected = 1,
		options = {true, false},
		options_text = {"Enabled","Disabled"},
		help = "Allows you to switch your current piece with one in temporary storage",
	},
	{
		text = "Rotation System",
		init = function(self)
			self.options = {}
			for i,j in pairs(rot_system) do
				table.insert(self.options,i)
			end
			self.selected = getindex(self.options, settings.conf.rot_system)
		end,
		onchange = function(self)
			settings.conf.rot_system = self.options[self.selected]
		end,
		selected = 1,
		options = {},
		help = "Different games have different centers of rotation for each piece. SRS is the modern one.",
	},
	{
		text = "Hard Drop",
		init = function(self)
			if settings.conf.harddrop then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			settings.conf.harddrop = self.options[self.selected]
		end,
		selected = 1,
		options = {true, false},
		options_text = {"Enabled","Disabled"},
		help = "An instantanious drop as far as the piece can go",
	},
	{
		text = "Softlock",
		init = function(self)
			if settings.conf.softlock then
				self.selected = 1
			else
				self.selected = 2
			end
		end,
		onchange = function(self)
			settings.conf.softlock = self.options[self.selected]
		end,
		selected = 2,
		options = {true, false},
		options_text = {"Enabled","Disabled"},
		help = "Resets soft drop when the piece locks, so you have to press down again when the new piece spawns to start another soft drop. Also disallows pressing any 2 directions at the same time.",
	},
	{
		text = "Wall Kick System",
		init = function(self)
			self.selected = getindex(self.options, settings.conf.kicks)
		end,
		onchange = function(self)
			settings.conf.kicks = self.options[self.selected]
		end,
		selected = 1,
		-- TODO make these not hard-coded
		options = {"permissive", "none", "atari"},
		options_text = {"Permissive", "None", "Atari"},
		help = "How pieces will move if they can't rotate",
	},
}

return items