-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/city/"


local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(0, i*32, 32,32, 32,224) end

local canvas_bg = lg.newCanvas(360,240)
local canvas_norm = lg.newCanvas(360,240)

local timer = 0
local cloudiness = 0
local cloudiness_target = 0
local cloudiness_timer = 0
local daytime = 0
local palette = love.image.newImageData(folder .. "palette.png")

local fonttext = "0123456789"
local fontquad = {}
for x = 0,9 do
	fontquad[x] = lg.newQuad(x*6, 0, 6, 7, 60, 7)
end
-- hard-coded right justified
local function drawfont(text, x, y)
	local len = #text
	for i = 1,len do
		lg.draw(style.tex.font, fontquad[tonumber(text:sub(len-i+1, len-i+1))], x - i*6, y)
	end
end

local shader_norm = lg.newShader [[
	extern Image map;
	extern float fac;
	vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
	{
		vec3 mapcol = Texel(map, uv).rgb;
		return Texel(tex,uv.xy + mapcol.b*((0.5 - mapcol.rg)/(8.0/fac)*ceil(mapcol.r + mapcol.g)));
	}
]]

local shader_lights = lg.newShader [[
extern Image map;
extern float fac;
vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
{
	vec4 col = Texel(tex,uv).rgba;
	float alph = Texel(map,uv).r;
	if (alph < fac) {
		return col;
	} else {
		return vec4(0,0,0,0);
	}
}
]]

local shader_clouds = lg.newShader [[
extern Image map;
extern float fac;
extern float scroll;
vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
{
	vec2 uv2 = mod(vec2(
		(uv.x-.5)*(uv.y+0.3)+scroll,
		uv.y
	),vec2(1.0));
	vec4 col = Texel(tex,uv2).rgba;
	float alph = Texel(map,uv2).r;
	return vec4(col.rgb*color.rgb, pow(alph, -fac*2.0)-1.0);
}
]]

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "nes",
	scoring = "minobueno",
	randomizer = "modern",
	lockdelay = 60,
	lockdelay_soft = 20,
	lockreset = "infinity",

	ARE = 0,
	clear_delay = 10,
	gameover_check = "oncollision",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 8,
	das_period = 3,

	softdrop_delay = 0,
	softdrop_period = 1,
	softlock = false,

	ghost = true,
	rot_system = "srs",
	kicks = "permissive",
	hold = true,
	harddrop = true,
	softdrop = true,
	spinny = true,
	smoothfall = "late",
	smoothslide = 2,
	piece_gfx = "city",
	piece_sfx = "city",
	album = "Remixes",
	song = "Dance of the Sugar Plum Fairy",
	next = true,
},

size = {
	w = 360,
	h = 240,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 140,
	y = 40,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-1}, -- I
	{ 3, 0}, -- J
	{ 3, 0}, -- L
	{ 3,-1}, -- O
	{ 3, 0}, -- S
	{ 3, 0}, -- T
	{ 3, 0}, -- Z
},
gameover_time = 3*60,

gameover = function()
	playstylesfx("gameover")
end,

init = function(vars)
	shader_lights:send("map", style.tex.noise)
	shader_clouds:send("map", style.tex.clouds_mask)
end,

update = function(dt)
	timer = timer + dt
	-- 300-second long days
	daytime = (daytime+dt/300)%1

	-- randomly choose a target cloudiness every 5 seconds
	cloudiness_timer = cloudiness_timer - dt
	if cloudiness_timer < 0 then
		cloudiness_target = math.random(0,10)*.1
		-- 1 in 11 chance to start rain
		if cloudiness_target == 1 then
			cloudiness_timer = 40
			cloudiness_target = 2
		else
			cloudiness_timer = 15
			-- prefer less clouds
			cloudiness_target = cloudiness_target^3
		end
	end
	cloudiness = cloudiness + (cloudiness_target - cloudiness)*dt*.1
	--cloudiness = 1
	if cloudiness > 1 and settings.stylesfx then
		if not style.sfx.rain:isPlaying() then
			style.sfx.rain:play()
		end
		style.sfx.rain:setVolume(max(0, (cloudiness - 1.5)*0.5))
	else
		style.sfx.rain:setVolume(0)
	end
end,

draw = function(vars, canvas)
	local held = vars.held
	local board = vars.board
	local block = vars.block

	lg.setCanvas(canvas_bg)

	-- stars
	lg.draw(style.tex.stars, 0, floor(0 - (timer)%360))
	lg.draw(style.tex.stars, 0, floor(360 - (timer)%360))

	-- sky
	lg.setColor(palette:getPixel(floor(daytime*256), 2))
	--lg.rectangle("fill", 0, 0, 800, 600)
	lg.draw(style.tex.sky)

	--~ lg.setColor(1,1,1,min(cloudiness, 1))
	--~ lg.draw(style.tex.clouds_day, floor(0- (timer*2)%512), 0)
	--~ lg.draw(style.tex.clouds_day, floor(512 - (timer*2)%512), 0)
	--~ lg.setColor(1,1,1,1)
	--~ lg.draw(style.tex.fg_day)


	-- clouds
	lg.setColor(palette:getPixel(floor(daytime*256), 3))
	lg.setShader(shader_clouds)
	shader_clouds:send("fac", cloudiness)
	--lg.draw(style.tex.clouds, floor(0- (timer*2)%512), 0)
	--lg.draw(style.tex.clouds, floor(512 - (timer*2)%512), 0)
	shader_clouds:send("scroll", timer*.005)
	lg.draw(style.tex.clouds)
	shader_clouds:send("scroll", timer*.01)
	lg.draw(style.tex.clouds)
	lg.setShader()
	lg.setColor(1,1,1)

	-- night foreground
	lg.draw(style.tex.fg_night)

	-- light
	if daytime < 0.1 or daytime > 0.6 then
		lg.setBlendMode("add")
		lg.setColor(1,1,1, 0.95 + math.random(0,10)*0.05 )
		lg.draw(style.tex.fg_night_light)
		lg.setColor(1,1,1,1)
		lg.setBlendMode("alpha")
	end

	-- day foreground
	lg.setColor(palette:getPixel(floor(daytime*256), 1))
	lg.draw(style.tex.fg_day)
	lg.setColor(1,1,1)

	-- fog
	if cloudiness > 1 then
		lg.setColor(1,1,1, (cloudiness-1))
		lg.draw(style.tex.fog)
		lg.setColor(1,1,1,1)
	end

	-- city lights
	local r,g,b,a = palette:getPixel(floor(daytime*256), 1)
	lg.setShader(shader_lights)
	shader_lights:send("fac", 1-a)
	lg.setBlendMode("add")
	lg.draw(style.tex.fg_citylights)
	lg.setBlendMode("alpha")
	lg.setShader()

	-- color things a little bit to match the sky
	-- some piece renderers will overwrite the color info
	lg.setColor(palette:getPixel(floor(daytime*256), 4))
	lg.draw(style.tex.overlay)


	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	lg.setScissor(
		style.board.x,
		style.board.y,
		style.board.w*8,
		style.board.h*8
	)
	drawpieces.board(board, block, vars.level)

	-- draw pieces
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.setScissor()
	lg.pop()

	-- draw score, lines
	lg.setColor(palette:getPixel(floor(daytime*256), 4))
	drawfont(tostring(vars.score), 136, 48)
	drawfont(tostring(vars.lines), 136, 64)

	if vars.next[1] ~= -1 and settings.conf.next then
		lg.draw(
			style.tex.next,
			nextquad[vars.next[1]],
			221,
			40
		)
	end

	-- hold piece
	if held.hold ~= -1 and settings.conf.hold then
		lg.draw(
			style.tex.next,
			nextquad[held.hold],
			180-16,
			8
		)
	end

	lg.setColor(1,1,1)

	-- draw halo from light in front of everything else
	if daytime < 0.1 or daytime > 0.6 then
		lg.draw(style.tex.halo)
	end

	-- generate rain normal map
	lg.setCanvas(canvas_norm)
	lg.draw(style.tex.rain, 0, floor((timer*512)%512))
	lg.draw(style.tex.rain, 0, floor((timer*512)%512)-512)
	shader_norm:send("map", canvas_norm)

	lg.setCanvas(canvas)

	if cloudiness > 1.5 then
		shader_norm:send("fac", (cloudiness-1.5)*0.2)
		lg.setShader(shader_norm)
		lg.draw(canvas_bg)
		lg.setShader()
	else
		shader_norm:send("fac", 0)
		lg.draw(canvas_bg)
	end
	--lg.print(vars.score, 0, 16)
	--~ lg.print("cloudiness        =" .. cloudiness, 0, 24)
	--~ lg.print("cloudiness_target =" .. cloudiness_target, 0, 32)
	--~ lg.print("cloudiness_timer  =" .. cloudiness_timer, 0, 40)


end,

}
