-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/electronika/"

local exetime = 0 -- timer, except "timer" "time" and "clock" are already taken...

local fontquad = {}
for x = 0,11 do
	fontquad[x+1] = lg.newQuad(x*10, 0, 10, 11, 120, 11)
end
local fonttext = " *0123456789"
local function drawfont(text, x,y, color)
	if color then lg.setColor(color) end
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char, nil, true)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ", " .. string.byte(text:sub(i,i)) .. ")")
		end
		if fontquad[pos] then
			lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
		end
	end
	if color then lg.setColor(1,1,1) end
end

local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(0, i*44, 80, 44,80,308) end

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {
	levels = "nes",
	scoring = "electronika",
	randomizer = "nes",
	lockdelay = 0,
	lockdelay_soft = 0,
	lockreset = "none",

	ARE = 0,
	clear_delay = 0,
	gameover_check = "oncollision",
	delete_above_field = true,
	allow_above_field = false,

	das_delay = 16,
	das_period = 6,

	softdrop_delay = 0,
	softdrop_period = 0,
	softlock = true,

	ghost = false,
	rot_system = "electronika",
	kicks = "none",
	hold = false,
	harddrop = true,
	softdrop = false,
	spinny = false,
	smoothfall = false,
	smoothslide = 0,
	piece_gfx = "electronika",
	piece_sfx = "electronika",
	next = true,
},

size = {
	w = 800,
	h = 275,
},
size_aspectcorrect = { -- TODO idk what this should actually be
	w = 800,
	h = 600,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 270,
	y = 22,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-2}, -- I
	{ 3,-1}, -- J
	{ 3,-1}, -- L
	{ 3,-1}, -- O
	{ 3,-1}, -- S
	{ 3,-1}, -- T
	{ 3,-1}, -- Z
},
gameover_time = 3*60,

update = function(dt)
	exetime = exetime + dt
end,

draw = function(vars, canvas)
	local block = vars.block
	local board = vars.board
	local held = vars.held

	lg.setCanvas(canvas)

	if settings.stylesettings.electronika then
		-- black BG
		if not settings.stylesettings.electronika.transparent then
			lg.setColor(0,0,0)
			lg.rectangle("fill", 0, 0, 800, 275)
		else
			lg.clear()
		end

		local c = settings.stylesettings.electronika.color
		if c == "Green" then
			lg.setColor(0,1,0)
		elseif c == "Amber" then
			lg.setColor(1,3/4,0)
		elseif c == "Blue" then
			lg.setColor(1/4,0.5,1)
		else
			lg.setColor(1,1,1)
		end
	end

	lg.draw(style.tex.bg)
	drawfont(num(vars.score%1000, 2, " "), 100,44)
	-- TODO I have no idea how electronika tetris handles loads of these "1000 points" dots, so I'm capping it at 8
	for i = 1,min(floor(vars.score/1000), 8) do
		drawfont("*", 10,55)
	end

	-- next piece
	if vars.next[1] ~= -1 and settings.conf.next then
		local tex = style.tex.next
		if settings.conf.rot_system == "srs" then
			tex = style.tex.next_srs
		end
		lg.draw(
			tex,
			nextquad[vars.next[1]],
			170,
			121
		)
	end

	if game.state == "gameover" then
		lg.draw(style.tex.gameover, 0, 176)
	end

	-- time of "running program"
	drawfont(num(floor(exetime)%60, 1,"0"), 780,0)
	drawfont(num(floor(exetime/60)%60, 1,"0"), 750,0)
	-- just in case a game lasts over 99 hours
	drawfont(num(floor(exetime/3600)%100, 1,"0"), 720,0)

	lg.push();lg.translate(style.board.x, style.board.y)
	-- because electronika pieces are 20x11 and not the usual 8x8
	lg.scale(20/8, 11/8)
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()

end,

}