-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/gb/"

local palette = load_palette(folder.."palette.png")


local fonttext = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ.-*"
local fontquad = {}
for x = 0,15 do for y = 0,3 do
	fontquad[x+1 + y*16] = lg.newQuad(x*8, y*8, 8, 8, 320, 8)
end end
local function drawfont(text, x,y, color)
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
	end
end


local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(i*32, 0, 32, 32,224,32) end

local canvas_main = lg.newCanvas(160,144)
local canvas_board = lg.newCanvas(160,144)



-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "gameboy",
	scoring = "gameboy",
	randomizer = "nes",
	lockdelay = 0,
	lockdelay_soft = 0,
	lockreset = "none",

	ARE = 2,
	clear_delay = 80,
	gameover_check = "oncollision",
	delete_above_field = true,
	allow_above_field = true,

	das_delay = 24,
	das_period = 8,

	softdrop_delay = 3,
	softdrop_period = 3,
	softlock = true,

	ghost = false,
	rot_system = "gameboy",
	kicks = "none",
	hold = false,
	harddrop = false,
	softdrop = true,
	spinny = false,
	smoothfall = false,
	smoothslide = 0,
	piece_gfx = "gameboy",
	piece_sfx = "gameboy",
	album = "Nintendo's Gameboy",
	song = "A",
	next = true,
},

size = {
	w = 160,
	h = 144,
},

board = {
	-- board size
	w = 10,
	h = 18,
	-- board offset
	x = 16,
	y = 0,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-1}, -- I
	{ 3, 0}, -- J
	{ 3, 0}, -- L
	{ 3, 0}, -- O
	{ 3, 0}, -- S
	{ 3, 0}, -- T
	{ 3, 0}, -- Z
},
gameover_time = 3*60,

gameover = function()
	playstylesfx("gameover")
end,


draw = function(vars, canvas)
	local board = vars.board
	local block = vars.block
	local held = vars.held

	lg.setCanvas(canvas_main)
	lg.draw(style.tex.bg)

	-- draw numbers
	drawfont(num(min(vars.score,999999), 4," "), 112, 24)
	drawfont(num(min(vars.level,99), 3," "), 112, 56)
	drawfont(num(min(vars.lines,999), 3," "), 112, 80)

	-- next piece
	if vars.next[1] ~= -1 and settings.conf.next then
		local tex = style.tex.next
		if settings.conf.rot_system == "srs" then
			tex = style.tex.next_srs
		end
		lg.draw(
			tex,
			nextquad[vars.next[1]],
			120,
			104
		)
	end

	lg.setCanvas(canvas_board)
	lg.clear()
	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()

	-- get palette colors
	local p = settings.stylesettings.gb.palette; p = p or 1

	if game.state == "clearing" then
		-- line clear animation
		lg.setColor(
			palette[p-1][2][1],
			palette[p-1][2][2],
			palette[p-1][2][3]
		)
		for i = 0,style.board.h-1 do
			if vars.to_clear[i] and (vars.timer)%20 > 10 then
				lg.rectangle("fill",
					style.board.x,
					style.board.y + i * 8,
					style.board.w*8,
					8
				)
			end
		end
		lg.setColor(1,1,1)
	end

	lg.setShader(shader.palette)
	apply_pallete(palette, p-1)

	if game.state == "gameover" then
		for x = 0,style.board.w-1 do
			for y = 0,style.board.h-1 do
				if y + 160 > vars.timer then
					lg.draw(style.tex.gameover, style.board.x+x*8, style.board.y+y*8)
				end
			end
		end
	end


	lg.setCanvas(canvas)
	lg.draw(canvas_main)
	lg.setShader()
	lg.draw(canvas_board)

end,

}