-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/lockjaw/"

local fonttext = "0123456789"
local fontquad = {}
for x = 0,15 do
	fontquad[x+1] = lg.newQuad(x*16, 0, 16, 24, 160, 24)
end
local function drawfont(text, x,y)
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*16 - 16, y)
	end
end

local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(0,i*32, 32,32, 32,224) end

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "nes", -- TODO
	scoring = "lockjaw", -- TODO
	randomizer = "small", -- TODO
	lockdelay = 60, -- TODO
	lockdelay_soft = 60,
	lockreset = "infinity",

	ARE = 0,
	clear_delay = 10,
	gameover_check = "oncollision",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 10,
	das_period = 1,

	softdrop_delay = 10,
	softdrop_period = 1,
	softlock = false,

	ghost = true,
	rot_system = "srs",
	kicks = "permissive",
	hold = true,
	harddrop = true,
	softdrop = true,
	spinny = false,
	smoothfall = "early",
	smoothslide = 0,
	piece_gfx = "lockjaw",
	piece_sfx = "lockjaw",
	album = "Lockjaw",
	song = "BWV 814",
	next = true,
},

size = {
	w = 800,
	h = 600,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 24,
	y = 72,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-2}, -- I
	{ 3,-2}, -- J
	{ 3,-2}, -- L
	{ 3,-3}, -- O
	{ 3,-2}, -- S
	{ 3,-2}, -- T
	{ 3,-2}, -- Z
},
gameover_time = 1*60,

gameover = function()
	playstylesfx("gameover")
end,

draw = function(vars, canvas)
	local held = vars.held
	local board = vars.board
	local block = vars.block

	lg.setCanvas(canvas)

	local offset = 0
	if settings.stylesettings.lockjaw then
		offset = settings.stylesettings.lockjaw.offset
	end

	lg.draw(style.tex.bg)

	lg.draw(style.tex.board, offset, 0)
	drawfont(tostring(vars.lines), 175 + offset, 11)
	drawfont(tostring(vars.score), 184 + offset, 41)

	-- hold piece
	-- TODO make black and white when you can't hold again
	if held.hold ~= -1 and settings.conf.hold then
		local y = 0; if held.hold == 3 then y = y - 16 end -- o-piece alignment
		lg.draw(
			style.tex.next,
			nextquad[held.hold],
			24 + offset,
			32 + y,
			0,
			2
		)
	end

	lg.push();lg.translate(style.board.x + offset, style.board.y)
	-- next pieces
	lg.scale(3)
	for i = 0,5 do
		local y = (56/3)*i; if vars.next[i+1] == 3 then y = y - 8 end -- o-piece alignment
		if vars.next[i+1] ~= -1 then
			lg.draw(style.tex.next, nextquad[vars.next[i+1]], 264/3, y)
		end
	end
	lg.setScissor(style.board.x + offset, style.board.y, 24*10, 24*20)
	-- draw board
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end

	if game.state == "gameover" then
		for y = 0,style.board.h-1 do
			local t = style.gameover_time - vars.timer
			lg.setColor(0,0,0)
			lg.rectangle("fill", 0, y*block.size, style.board.w*block.size, max(t*.5 - y,0))
			lg.setColor(1,1,1)
		end
	end
	lg.pop()
	lg.setScissor()

end,

}