-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/nes/"

local palette = load_palette(folder.."palette.png")

local fonttext = "0123456789      $ABCDEFGHIJKLMNOPQRSTUVWXYZ      !\"    '()  ,-./"
local fontquad = {}
for x = 0,15 do for y = 0,3 do
	fontquad[x+1 + y*16] = lg.newQuad(x*8, y*8, 8, 8, 128, 32)
end end
local function drawfont(text, x,y, orange)
	if orange then lg.setColor(224/255, 80/255, 0) end
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
	end
	lg.setColor(1,1,1)
end


local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(i*32, 0, 32, 32,224,32) end

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {
	levels = "nes",
	scoring = "nes",
	randomizer = "nes",
	lockdelay = 0,
	lockdelay_soft = 0,
	lockreset = "none",

	ARE = 18,
	clear_delay = 20,
	gameover_check = "oncollision",
	delete_above_field = true,
	allow_above_field = true,

	das_delay = 16,
	das_period = 6,

	softdrop_delay = 2,
	softdrop_period = 2,
	softlock = true,

	ghost = false,
	rot_system = "nes",
	kicks = "none",
	hold = false,
	harddrop = false,
	softdrop = true,
	spinny = false,
	smoothfall = false,
	smoothslide = 0,
	piece_gfx = "nes",
	piece_sfx = "nes",
	album = "Nintendo's NES",
	song = "A",
	next = true,
},

size = {
	w = 256,
	h = 224,
},
size_aspectcorrect = {
	w = 224*(4/3),
	h = 224,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 96,
	y = 48,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-2}, -- I
	{ 3,-1}, -- J
	{ 3,-1}, -- L
	{ 3,-1}, -- O
	{ 3,-1}, -- S
	{ 3,-1}, -- T
	{ 3,-1}, -- Z
},
gameover_time = 3*60,

gameover = function()
	playstylesfx("gameover")
end,

draw = function(vars, canvas)
	local block = vars.block
	local board = vars.board
	local held = vars.held

	lg.setCanvas(canvas)
	lg.clear() -- needed because the style canvas is transparent

	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()
	-- draw BG in front to cover pieces
	lg.setShader(shader.palette)
		apply_pallete(palette, vars.level%(#palette+1))
		if settings.conf.hold then
			lg.draw(style.tex.bg_hold)
		else
			lg.draw(style.tex.bg)
		end

		local holdtex = style.tex.next
		if settings.conf.rot_system == "srs" then
			holdtex = style.tex.next_srs
		end
		if vars.next[1] ~= -1 and settings.conf.next then
			-- the next piece is drawn a bit down to make room for the hold piece, when it's enabled
			if settings.conf.hold then
				lg.draw(
					holdtex,
					nextquad[vars.next[1]],
					192,
					120
				)
			else
				lg.draw(
					holdtex,
					nextquad[vars.next[1]],
					192,
					112
				)
			end
		end

		-- hold piece
		if held.hold ~= -1 and settings.conf.hold then
			lg.draw(
				holdtex,
				nextquad[held.hold],
				192,
				64
			)
		end
	lg.setShader()

	if vars.lastclear == 4 and game.state == "clearing" then
		lg.setColor(1,1,1, ceil((vars.timer/4)%1-0.5) )
		if settings.conf.hold then
			lg.draw(style.tex.flash_hold)
		else
			lg.draw(style.tex.flash)
		end
		lg.setColor(1,1,1,1)
	end

	drawfont(num(min(vars.lines,999), 2,"0"), 152, 24)
	if settings.conf.hold then
		drawfont(num(min(vars.score,999999), 5,"0"), 192, 32)
		drawfont( num(min(vars.level,99), 1,"0") , 208, 176)
	else
		drawfont("000000", 192, 40)
		drawfont(num(min(vars.score,999999), 5,"0"), 192, 64)
		drawfont( num(min(vars.level,99), 1,"0") , 208, 168)
	end
	local pcount = vars.pieces --  TODO isnt this pointless?
	drawfont(num(pcount[6] ,2,"0"), 48, 96, true)
	drawfont(num(pcount[2] ,2,"0"), 48, 112, true)
	drawfont(num(pcount[7] ,2,"0"), 48, 128, true)
	drawfont(num(pcount[4] ,2,"0"), 48, 144, true)
	drawfont(num(pcount[5] ,2,"0"), 48, 160, true)
	drawfont(num(pcount[3] ,2,"0"), 48, 176, true)
	drawfont(num(pcount[1] ,2,"0"), 48, 192, true)

	if game.state == "clearing" then
		lg.setColor(0,0,0,1)
		for i = 0,style.board.h-1 do
			if vars.to_clear[i] then
				local size = 5 - floor(vars.timer/4)
				lg.rectangle("fill",
					style.board.x + 40 - 8*size,
					style.board.y+8*i,
					8*size*2,
					8
				)
			end
		end
		lg.setColor(1,1,1,1)
	elseif game.state == "gameover" then
		lg.setShader(shader.palette)
		for x = 0,style.board.w-1 do
			for y = 0,style.board.h-1 do
				if 120 - y*4 > vars.timer then
					lg.draw(style.tex.gameover, style.board.x+x*8, style.board.y+y*8)
				end
			end
		end
		lg.setShader()
	end

end,

}