-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/rain/"

local nextquad = {}
for i = 0,6 do nextquad[i] = lg.newQuad(0, i*200, 200, 200,200,1400) end

local fontquad = {}
for i = 0,9 do
	fontquad[tostring(i)] = lg.newQuad(i*25, 0, 25, 32, 250, 32)
end
local function drawfont(text, x,y)
	for i = 1,#text do
		local char = text:sub(i, i)
		lg.draw(style.tex.font, fontquad[char], x + i*24 - 8, y)
	end
end

local quad_prop = {}
for i = 0,5 do
	quad_prop[i] = lg.newQuad(i*200, 0, 200, 400, 1200, 400)
end

local canvas_main = lg.newCanvas(400,800)
local canvas_drip = lg.newCanvas(400,800)
local canvas_norm = lg.newCanvas(400,800)
local canvas_board = lg.newCanvas(400,800)
local canvas_clouds = lg.newCanvas(400,800)

local timer = 0
local cloudiness = 2
local raininess = 1
local cloudiness_timer = 60
local cloudiness_target = 2
local daytime = 0.25
local palette = love.image.newImageData(folder .. "palette.png")

-- time, in seconds, for lightning strike effects relative to sfx position
local lightning = {
	21.2,
	68.5,
	114.8,
	153.4,
	240,
	250.6,
	326.8
}
local lightning_last = #lightning -- last played effect, so we know when to play another
local lightning_countdown = 0

-- things specifically for the rain effect
local dripsound = 0
-- 8x16 rectangle
local rect = {}
local rectsize = 8*16
for i = 0,rectsize-1 do rect[i] = {} end

local shader_env = lg.newShader [[
	extern Image map;
	extern vec2 offset;
	vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
	{
		vec4 uv2;
		vec4 col;
		uv2 = Texel(map, uv);
		//col = Texel(tex, uv2.rg/uv2.b);
		col = Texel(tex, uv2.rg);
		return col;
	}
]]
local shader_norm = lg.newShader [[
	extern Image norm;
	vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
	{
		vec4 normcol = Texel(norm, uv.xy);
		vec3 bgcol = Texel(tex,
			vec2(
				uv.x + (
					normcol.r*.1
				) - .05,

				uv.y + (
					normcol.g*.1
				) - .05
			)
		).xyz;


		return vec4(bgcol, 1.0);

	}
]]
local shader_hack = lg.newShader [[
	extern Image map;
	vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
	{
		vec3 hack = Texel(map, uv).rgb;
		vec2 youvee = vec2(
			hack.r/hack.b,
			1.0 - hack.g/hack.b
		);
		vec3 col = Texel(tex, youvee).rgb;
		return vec4(col, hack.b)*color;
	}
]]

local shader_stormclouds = lg.newShader [[
extern Image map;
extern float fac;
vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
{
	vec4 col = Texel(tex,uv).rgba;
	float alph = Texel(map,uv).r;
	//return vec4(col.rgb, Texel(map,uv).r*fac + fac)*color;
	return vec4(col.rgb,
		clamp(fac/alph,0.0,1.0) - ((1.0 - fac)*2.0)
	)*color;
}
]]

local shader_clouds = lg.newShader [[
extern Image map;
extern float fac;
extern float scroll;
vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
{
	vec2 uv2 = vec2(
		mod( ((uv.x - 0.5)/tan(uv.y)) / 4.0 + mod(scroll,1.0),1.0),
		1.0-uv.y
	);
	vec4 col = Texel(tex,uv2).rgba;
	float alph = Texel(map,uv2).r;
	return vec4(
		col.rgb*color.rgb,
		//pow(fac,alph*4.0) - clamp(3.0-uv.y*4, 0.0, 1.0)
		clamp(fac/alph,0.0,1.0) - (1.0-fac) - (3.0-uv.y*4.0)
	);
}
]]

-- gets correct color from pallete, including interpolation
local getDayColor = function(row)
	return colormix(
		{palette:getPixel(floor(daytime*256), row)},
		{palette:getPixel(floor(daytime*256 + 1)%255, row)},
		(daytime*256)%1
	)
end

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "nes",
	scoring = "minobueno",
	randomizer = "modern",
	lockdelay = 60,
	lockdelay_soft = 20,
	lockreset = "infinity",

	ARE = 0,
	clear_delay = 10,
	gameover_check = "oncollision",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 10,
	das_period = 3,

	softdrop_delay = 8,
	softdrop_period = 2,
	softlock = false,

	ghost = true,
	rot_system = "srs",
	kicks = "permissive",
	hold = true,
	harddrop = true,
	softdrop = true,
	spinny = true,
	smoothfall = "early",
	smoothslide = 2,
	piece_gfx = "city",
	piece_sfx = "city",
	album = "Bach",
	song = "BWV 776",
	next = true,
},

size = {
	w = 1920,
	h = 1080,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 760,
	y = 56,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-1}, -- I
	{ 3, 0}, -- J
	{ 3, 0}, -- L
	{ 3,-1}, -- O
	{ 3, 0}, -- S
	{ 3, 0}, -- T
	{ 3, 0}, -- Z
},
gameover_time = 4*60,

gameover = function()
	playstylesfx("gameover")
end,

init = function()
	style.tex.hack:setFilter("linear", "linear")
	shader_hack:send("map", style.tex.hack)
	shader_clouds:send("map", style.tex.clouds_alpha)
	shader_stormclouds:send("map", style.tex.clouds_alpha)
	-- initialize normal map canvas
	canvas_drip:renderTo(function()
		lg.setColor(0.5,0.5,1,1)
		lg.rectangle("fill", 0, 0, 400, 800)
		lg.setColor(1,1,1,1)
	end)
	-- maybe a little lazy to add reverb to everything
	for i in pairs(style.sfx) do
		style.sfx[i]:setEffect("reverb1")
	end

end,

update = function(dt)
	-- START DRIPS
	--speed = max(speed - dt,1)
	-- add random drops, play SFX
	if raininess > 0 then
		for i = 1,2 do
			if rand(1,300) < raininess * 100 then
				local box = rand(0,rectsize-1)
				local x = box%8
				local y = floor(box/8)
				local x2 = x + rand(0,31)/32
				local y2 = y + rand(0,31)/32
				local s = rand(1, 8)/16
				table.insert(rect[box],{
					x = x2,
					y = y2,
					size = s
				})
				if settings.stylesfx then
					dripsound = (dripsound + 1)%5
					local src = style.sfx[tostring(dripsound + 1)]
					src:setVolume(rand(1,100)*.01*s)
					src:setPitch(rand(1,100)*.001+1)
					src:setPosition((x2/8 - .5)*.1, (y2/16 - .5)*.1, 1)
					src:play()
				end
			end
		end

	else -- slowly shrink drops
		for i,box in pairs(rect) do for k, drop in pairs(box) do
			drop.size = drop.size - dt * 0.25
			if drop.size <= 0 then table.remove(box, k) end
		end end
	end

	-- move closer together
	for i,rec in pairs(rect) do
		local neighbors = {rect[i]}
		local function ins(ert)
			if ert ~= nil then table.insert(neighbors, ert) end
		end
		ins(rect[i + 8])
		if i%8 > 0 then
			ins(rect[i - 1])
			ins(rect[i - 1 + 8])
		end
		if i%8 < 7 then
			ins(rect[i + 1])
			ins(rect[i + 1 + 8])
		end
		for j,drop in pairs(rec) do
			for f,box in pairs(neighbors) do
				for k,odrop in pairs(box) do
					if odrop ~= drop and odrop.y >= drop.y then
						local x1,x2,y1,y2 = drop.x, odrop.x, drop.y, odrop.y
						local d = dist(x1,y1,x2,y2)
						local s = 1/d^2 -- inverse square

						if d < 0.05 then -- combine super close
							drop.size = drop.size + odrop.size
							table.remove(box, k)
						elseif d < 0.5 then
							if drop.size > 2 and d < 0.2 then -- if the drop is falling fast, just combine them
								drop.x = x1 + (x2 - x1) * .5
								drop.y = y2 + (y2 - y1) * .5
								drop.size = drop.size + odrop.size
								table.remove(box,k)
							else -- move them closer
								drop.x = x1 + (x2 - x1)*s * dt * .125
								drop.y = y1 + (y2 - y1)*s * dt * .125
							end
						end
					end
				end
			end
		end
	end

	--find out what drops need to be moved into a new spot
	for i,rec in pairs(rect) do for j,drop in pairs(rec) do
		local spot = math.floor(drop.x)%8 + math.floor(drop.y) * 8
		if spot > 8*16 - 1 or drop.x < 0 then
			table.remove(rec, j)
		elseif spot ~= i then
			table.remove(rec, j)
			table.insert(rect[spot], drop)
		elseif drop.size > 2.0 then
			drop.y = drop.y + dt * 6
			drop.x = drop.x + dt * (rand(0,7) - 4)*.5 - dt * 2
		end
	end end
	-- END DRIPS


	timer = timer + dt
	-- 300-second long days
	daytime = (daytime+dt/300)%1

	-- randomly choose a target cloudiness
	cloudiness_timer = cloudiness_timer - dt
	if cloudiness_timer < 0 then
		cloudiness_target = math.random(0,11)*.1
		-- 2 in 13 chance to start rain
		if cloudiness_target >= 1 then
			cloudiness_timer = 90
			cloudiness_target = 2
		else
			cloudiness_timer = 30
		end
	end
	cloudiness = cloudiness + (cloudiness_target - cloudiness)*dt*.05
	raininess = clamp((cloudiness - 1.5)*2, 0, 1)


	if cloudiness > 1 and settings.stylesfx then
		style.sfx.rain:setVolume(raininess)
	else
		style.sfx.rain:setVolume(0)
	end
	-- play sfx even when muted so we can calculate lightning effect
	if not style.sfx.rain:isPlaying() then
		style.sfx.rain:play()
		lightning_last = #lightning
	end

	if settings.stylesfx and settings.stylesettings.rain.prop then
		if not style.sfx.prop:isPlaying() then
			style.sfx.prop:play()
		end
	else
		style.sfx.prop:stop()
	end

	-- lightning
	if style.sfx.rain:tell() >= lightning[(lightning_last)%#lightning+1] and lightning_last ~= #lightning
	or lightning_last == #lightning and style.sfx.rain:tell() > lightning[1] and style.sfx.rain:tell() < lightning[#lightning] then
		lightning_last = (lightning_last%#lightning)+1
		lightning_countdown = 1
	else
		lightning_countdown = max(lightning_countdown - dt*3, 0)
	end
end,

draw = function(vars, canvas)
	local held = vars.held
	local board = vars.board
	local block = vars.block

	-- draw sky
	lg.setCanvas(canvas_clouds)

	lg.setColor(getDayColor(2))
	lg.draw(style.tex.sky)

	-- clouds
	lg.setColor(getDayColor(3))
	lg.setShader(shader_clouds)
	shader_clouds:send("fac", min(cloudiness, 1.0))
	shader_clouds:send("scroll", timer*.02)
	lg.draw(style.tex.clouds, 0, 400)
	shader_clouds:send("scroll", timer*.04)
	lg.draw(style.tex.clouds2, 0, 400)

	-- storm clouds
	lg.setShader(shader_stormclouds)
	shader_stormclouds:send("fac", clamp(cloudiness - 0.5, 0, 1))
	lg.draw(style.tex.clouds_storm, (daytime*24)%1*-1024, 0)
	lg.draw(style.tex.clouds_storm, (daytime*24)%1*-1024 + 1024, 0)
	if lightning_countdown > 0 and math.random(0,2) == 0 and raininess > 0 then
		lg.setColor(0.5,3/4,1, raininess)
		lg.setBlendMode("add")
		lg.draw(style.tex.clouds_storm, (daytime*24)%1*-1024, 0)
		lg.draw(style.tex.clouds_storm, (daytime*24)%1*-1024 + 1024, 0)
		lg.setBlendMode("alpha")
		lg.setColor(1,1,1)
	end
	lg.setShader()


	-- draw prop, also to cloud canvas, with cloud colors
	if settings.stylesettings.rain.prop then
		-- add a bit of shake when it rains, too
		lg.draw(style.tex.prop, quad_prop[math.random(0,5)], 0, 200 + math.random(0,1)*raininess)
	end

	lg.setColor(1,1,1)

	-- draw drips
	lg.setCanvas(canvas_main)
	lg.clear()
	for i,box in pairs(rect) do for k, drop in pairs(box) do
		local x,y,s = drop.x, drop.y, drop.size
		lg.draw(style.tex.norm, x*50, y*50, 0, sqrt(s)/32, nil, 256,256)
	end end

	--draw drip trail
	lg.setCanvas(canvas_drip)
	lg.draw(canvas_main)


	lg.setBlendMode("alpha", "premultiplied")
	lg.setColor(3/255,3/255,6/255,5/255)

	lg.rectangle("fill", 0, 0, 400, 800)

	lg.setBlendMode("alpha", "alphamultiply")
	lg.setColor(1,1,1,1)

	--lg.draw(style.tex.norm, 64, 192, 0, .25)

	lg.setCanvas(canvas_norm)
	lg.draw(canvas_drip)
	lg.draw(canvas_main)

	lg.setCanvas(canvas_board)
	shader_env:send("map", canvas_norm)
	shader_norm:send("norm", canvas_norm)

	-- draw distorted bg
	lg.setShader(shader_norm)
	lg.draw(canvas_clouds)
	-- draw env map shininess
	lg.setShader(shader_env)
	lg.draw(style.tex.env)
	lg.setShader()


	-- draw board
	lg.push();lg.scale(5)
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()

	-- set color to daytime color, mix with grey depending on raininess
	lg.setColor(colormix(getDayColor(4), {0.5,0.5,0.5}, raininess))

	lg.setCanvas(canvas)

	lg.draw(style.tex.main)
	lg.draw(canvas_board, style.board.x, style.board.y)
	lg.setBlendMode("add")
	lg.setShader(shader_hack)
	lg.draw(canvas_board,0,0,0, 1920/400, 1080/800)
	lg.setBlendMode("alpha")
	lg.setShader()

	lg.draw(canvas_board, style.board.x, style.board.y)

	lg.setColor(1,1,1,1)

	-- emissive things
	lg.setBlendMode("add")
	lg.draw(style.tex.light)
	drawfont(num(min(vars.score,99999999), 7,"0"), 188,328)
	drawfont(num(min(vars.lines,99999999), 7,"0"), 188,395)
	-- 1226,33
	if held.hold ~= -1 and settings.conf.hold then
		lg.draw(
			style.tex.next,
			nextquad[held.hold],
			1235,
			33
		)
	end
	if vars.next[1] ~= -1 and settings.conf.next then
		lg.draw(
			style.tex.next,
			nextquad[vars.next[1]],
			1235,
			384
		)
	end
	lg.setBlendMode("alpha")
end,

}