-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/sega/"

local fonttext = " !\"#$%&'()*+,-./0123456789:;<>=?ABCDEFGHIJKLMNOPQRSTUVWXYZ}{\\`~_@|"
local fontquad = {}
for x = 0,65 do for y = 0,1 do
	fontquad[x+1 + y*65] = lg.newQuad(x*8, y*8, 8, 8, 528, 16)
end end
local function drawfont(text, x,y, orange)
	offset = 0
	if orange then offset = 8 end
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
	end
end


local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(0, i*32, 32, 32,32,224) end

local line_quad = {}
local line_gfx = {0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6}
for i = 0,6 do line_quad[i] = lg.newQuad(i*8, 0, 8, 8,56,8) end

-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {
	levels = "sega_normal",
	scoring = "nes", -- TODO
	randomizer = "nes", -- TODO
	lockdelay = 30,
	lockdelay_soft = 0, -- TODO
	lockreset = "sega", -- TODO sega lock delay isn't actually implemented yet

	ARE = 30,
	clear_delay = 42,
	gameover_check = "oncollision",
	delete_above_field = true, -- this option doesn't even matter in default sega style
	allow_above_field = false,

	das_delay = 20,
	das_period = 1,

	softdrop_delay = 1, -- TODO
	softdrop_period = 1, -- TODO
	softlock = false,

	ghost = false,
	rot_system = "sega",
	kicks = "none",
	hold = false,
	harddrop = false,
	softdrop = true,
	spinny = false,
	smoothfall = false,
	smoothslide = 0,
	piece_gfx = "sega",
	piece_sfx = "sega", -- TODO
	album = "Sega", -- TODO
	song = "Arcade", -- TODO
	next = true,
},

size = {
	w = 320,
	h = 224,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 120,
	y = 32,
},

-- TODO
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-2}, -- I
	{ 3,-1}, -- J
	{ 3,-1}, -- L
	{ 3,-1}, -- O
	{ 3,-1}, -- S
	{ 3,-1}, -- T
	{ 3,-1}, -- Z
},

gameover_time = 13*60,

gameover = function()
	playstylesfx("gameover")
end,

draw = function(vars, canvas)
	local block = vars.block
	local board = vars.board
	local held = vars.held

	lg.setCanvas(canvas)

	local bg = settings.stylesettings.sega.bg or 1
	lg.draw(style.tex.bg[tostring(bg)])
	if settings.stylesettings.sega.trans then
		lg.draw(style.tex.board_trans, 112, 24)
	else
		lg.draw(style.tex.board, 112, 24)
	end

	drawfont("NEXT", 112,16)
	drawfont("SCORE", 56,24)
	drawfont(num(min(vars.score,99999), 4," "), 56,32) --TODO find actual max score
	drawfont("LINES", 56,48)
	drawfont(num(min(vars.lines,99999), 4," "), 56,56) --TODO find actual max lines
	drawfont("LEVEL", 56,72)
	drawfont(num(min(vars.level,99), 1," "), 80,80)

	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	lg.setScissor(120, 32, 80, 160)
		drawpieces.board(board, block, vars.level)
		if game.state == "playing" or game.state == "clearing" then
			drawpieces.pieces(vars, game.state)
		end

		-- line clear animation
		if game.state == "clearing" then
			for y = 0,style.board.h-1 do
				if vars.to_clear[y] then
					for x = 0,style.board.w-1 do
						lg.draw(
							style.tex.line_clear,
							line_quad[line_gfx[clamp(42-vars.timer,1,40)]],
							x*block.size,y*block.size
						)
					end
				end
			end
			if vars.timer < 20 then
				local bottom = style.board.h
				for i = bottom,1,-1 do
					bottom = i
					if vars.to_clear[i] then break end
				end
				-- TODO I know this hard-coded score thing isn't the best...
				local text = ""
				if vars.lastclear == 1 then
					text = " 100"
				elseif vars.lastclear == 2 then
					text = " 400"
				elseif vars.lastclear == 3 then
					text = " 900"
				elseif vars.lastclear == 4 then
					text = "2000"
				end
				drawfont(text, block.size * 3, bottom*block.size)
			end
		elseif game.state == "gameover" then
			for x = 0,style.board.w-1 do
				for y = 0,min( (13*60-vars.timer)*0.2, style.board.h) do
					if board[x][y] ~= -1 then
						lg.draw(
							style.tex.line_clear,
							line_quad[3],
							x*block.size,
							y*block.size
						)
					end
				end
			end
			if vars.timer < 13*60 - 220 then -- just a rough estimate, its good enough
				drawfont("GAME", 8*2, 8*8)
				drawfont("OVER", 8*3, 8*9)
			end
		end
	lg.setScissor()
	lg.pop()

	if vars.next[1] ~= -1 and settings.conf.next then
		-- the next piece is drawn a bit down to make room for the hold piece, when it's enabled
		local tex = style.tex.next
		if settings.conf.rot_system == "srs" then
			tex = style.tex.next_srs
		end
		lg.draw(
			tex,
			nextquad[vars.next[1]],
			144,
			0
		)
	end

	-- TODO
	-- hold piece
	if settings.conf.hold then
		drawfont("HOLD", 208, 24)
		if held.hold ~= -1 then
			lg.draw(
				style.tex.next,
				nextquad[held.hold],
				208,
				32
			)
		end
	end
end,

}
