-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/st2/"

local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(0,i*8, 16,8, 16,56) end

local fonttext = "0123456789 "
local fontquad = {}
for x = 1,11 do fontquad[x] = lg.newQuad((x-1)*8, 0, 8, 8, 88, 8) end
local function drawfont(text, x,y, color)
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
	end
end


local bgtime = 0
local bgquad = {}
for i = 0,7 do bgquad[i] = lg.newQuad(i*256, 0, 256, 240,256*8,240) end

local linequad = {}
for i = 0,3 do linequad[i] = lg.newQuad(i*8, 0, 8, 8, 32, 8) end


-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {
	levels = "nes", -- TODO
	scoring = "nes", -- TODO
	randomizer = "nes", -- TODO
	lockdelay = 30, -- or maybe its 29, idk
	lockdelay_soft = 0,
	lockreset = "none",

	ARE = 16, -- something like that, whatever
	clear_delay = 20, -- plus 2 frames for each line cleared
	gameover_check = "overtop",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 8,
	das_period = 2,

	softdrop_delay = 4, -- close enough
	softdrop_period = 1,
	softlock = false,

	ghost = false,
	rot_system = "bps",
	kicks = "none",
	hold = false,
	harddrop = false,
	softdrop = true,
	spinny = true,
	smoothfall = "late",
	smoothslide = 2,
	piece_gfx = "st2",
	piece_sfx = "st2",
	album = "Super Tetris 2 + Bombliss",
	song = "Tetris A-Type",
	next = true,
},

size = {
	w = 256,
	h = 240,
},
size_aspectcorrect = {
	w = 224*(4/3),
	h = 240,
},

board = {
	-- board size
	w = 10,
	h = 22,
	-- board offset
	x = 88,
	y = 31,
},

-- TODO
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-2}, -- I
	{ 3,-2}, -- J
	{ 3,-2}, -- L
	{ 3,-2}, -- O
	{ 3,-2}, -- S
	{ 3,-2}, -- T
	{ 3,-2}, -- Z
},

gameover_time = 120,

gameover = function()
	playstylesfx("gameover")
end,

update = function(dt)
	bgtime = (bgtime + dt*(60/16))%8
end,

draw = function(vars, canvas)
	local block = vars.block
	local board = vars.board
	local held = vars.held

	lg.setCanvas(canvas)

	lg.draw(style.tex.bg["1"], bgquad[floor(bgtime)])
	-- 4-line flash
	if game.state == "clearing" and vars.timer%2 == 1 and vars.timer > 5 and vars.lastclear == 4 then
		lg.rectangle("fill", 0, 0, 256, 240)
	end
	lg.setBlendMode("subtract")
	lg.draw(style.tex.shadow)
	lg.setBlendMode("alpha")
	lg.draw(style.tex.board)

	drawfont(num(min(vars.score,999999), 5," "), 16,184)
	drawfont(num(min(vars.lines,999999), 5," "), 192,184)
	drawfont(num(min(vars.level,99), 1," "), 216,40)
	drawfont("10000", 24,72)

	if vars.next[1] ~= -1 and settings.conf.next then
		lg.draw(
			style.tex.next,
			nextquad[vars.next[1]],
			120,
			19
		)
	end

	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	lg.setScissor(88, 31, 80, 176)
		drawpieces.board(board, block, vars.level)

		if game.state == "playing" or game.state == "clearing" then
			drawpieces.pieces(vars, game.state)
		end

		if game.state == "gameover" then
			for x = 0,style.board.w-1 do
				for y = style.board.h-1,max(vars.timer*.5-40,0),-1 do
					lg.draw(style.tex.gameover, x*block.size, y*block.size)
				end
			end
		elseif game.state == "clearing" then
			local tile = 3
			local t = 20 - vars.timer
			if t > 3 and t < 15 then
				if (vars.timer+3)%4 < 2 then tile = 0 end
			elseif t == 15 then
				tile = 1
			elseif t == 16 then
				tile = 2
			end
			print(tile)
			for y = 0,style.board.h-1 do if vars.to_clear[y] then
				for x = 0,style.board.w-1 do
					lg.draw(style.tex.lineclear, linequad[tile], x*block.size, y*block.size)
				end
			end end
		end
	lg.setScissor()
	lg.pop()

end,

}
