-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/tengen/"

local palette = load_palette(folder.."palette.png")

local fonttext = "=><-.c0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ "
local fontquad = {}
for x = 0,15 do for y = 0,3 do
	fontquad[x+1 + y*16] = lg.newQuad(x*8, y*8, 8, 8, 128, 24)
end end
local function drawfont(text, x,y, color)
	if color then lg.setColor(color) end
	for i = 1,#text do
		local char = text:sub(i, i)
		local pos = fonttext:find(char)
		if pos == nil then
			error("Tried to draw character that doesn't exist (" .. text:sub(i,i) .. ")")
		end
		lg.draw(style.tex.font, fontquad[pos], x + i*8 - 8, y)
	end
	if color then lg.setColor(1,1,1) end
end

local nextquad = {}
for i = 0,7 do nextquad[i] = lg.newQuad(i*32,0, 32,32, 224,32) end

-- piece count graph
local graphquad = {}
for i = 0,6 do
	graphquad[i+1] = lg.newQuad(i*8,0, 8,144, 56,4)
end

local dropscore = {
	timer = 0,
	score = 0,
	offset = 0,
}


-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "tengen",
	scoring = "tengen",
	randomizer = "nes",
	lockdelay = 0,
	lockdelay_soft = 0,
	lockreset = "none",

	ARE = 18,
	clear_delay = 20,
	gameover_check = "overtop",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 12,
	das_period = 6,

	softdrop_period = 2,
	softdrop_delay = 8,
	softlock = true,

	ghost = false,
	rot_system = "tengen",
	kicks = "atari",
	hold = false,
	harddrop = false,
	softdrop = true,
	spinny = false,
	smoothfall = false,
	smoothslide = 0,
	piece_gfx = "tengen",
	piece_sfx = "tengen",
	album = "Atari's NES",
	song = "Bradinsky",
	next = true,
},

size = {
	w = 256,
	h = 224,
},
size_aspectcorrect = {
	w = 224*(4/3),
	h = 224,
},

board = {
	-- board size
	w = 10,
	h = 20,
	-- board offset
	x = 16,
	y = 56,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 4,-4}, -- I
	{ 4,-3}, -- J
	{ 4,-3}, -- L
	{ 3,-3}, -- O
	{ 4,-3}, -- S
	{ 4,-3}, -- T
	{ 4,-3}, -- Z
},
gameover_time = 2.5*60,

dropscore = function(lowest_block, dscore)
	if settings.conf.scoring == "tengen" then
		dropscore.timer = 60
		dropscore.offset = lowest_block
		dropscore.score = dscore
	end
end,

gameover = function()
	playstylesfx("gameover")
end,

init = function(vars)

	style.tex.graph:setWrap("repeat","repeat")
	style.tex.graph_vs:setWrap("repeat","repeat")
	style.clearquad = {}
	for i = 0,6 do
		style.clearquad[i] = lg.newQuad(8*i, 0, 8, 8, 56, 8)
	end
end,

update = function(dt)
	if dropscore.timer > 0 then dropscore.timer = dropscore.timer - 1 end
end,

draw = function(vars, canvas)
	local block = vars.block
	local board = vars.board
	local held = vars.held

	lg.setCanvas(canvas)
	lg.clear() -- needed because the style canvas is transparent

	-- next piece
	if vars.next[1] ~= -1 and settings.conf.next then
		local tex = style.tex.next
		if settings.conf.rot_system == "srs" then
			tex = style.tex.next_srs
		-- TODO doing it this way means you can't have SRS graphics with VS tetris
		elseif settings.stylesettings.tengen.bg then
			tex = style.tex.next_vs
		end
		lg.draw(
			tex,
			nextquad[vars.next[1]],
			16,
			16
		)
	end

	-- draw pieces
	lg.push();lg.translate(style.board.x, style.board.y)
	drawpieces.board(board, block, vars.level)
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()

	-- tengen background is actually in front of the pieces
	if settings.stylesettings.tengen.bg then
		lg.draw(style.tex.bg_vs)
	else
		lg.draw(style.tex.bg)
	end

	-- general text
	drawfont(num(min(vars.lines,9999), 3," "), 72, 24)
	drawfont(num(min(vars.level,99), 1," ") , 88, 40)
	drawfont("000000", 176, 24)
	drawfont(num(min(vars.score, 999999), 5," "), 56, 8)

	-- soft drop bonus
	if dropscore.timer > 0 then
		drawfont(
			num(dropscore.score, 1," "),
			96,
			56+dropscore.offset*block.size,
			{1,0xde/0xff,0x9c/0xff}
		)
	end

	-- draw piece count
	--~ for i,k in ipairs {1,6,4,2,3,5,7} do
		--~ local p = min(vars.pieces[k],144)
		--~ lg.draw(style.tex.piececount, 158 + 8*i,199-p, 0, 1, p)
	--~ end
	for i,k in ipairs {1,6,4,2,3,5,7} do
		local p = min(vars.pieces[k],144)
		if settings.stylesettings.tengen.bg then
			lg.setScissor(168 + 8*i, 216 - p, 8, p)
			lg.draw(style.tex.graph_vs, graphquad[i], 168 + 8*i, 80)
		else
			lg.setScissor(160 + 8*i, 200 - p, 8, p)
			lg.draw(style.tex.graph, graphquad[i], 160 + 8*i, 56)
		end
	end
	lg.setScissor()

	if game.state == "clearing" then
		for i = 0,style.board.h-1 do
			if vars.to_clear[i] then
				for x = 0,9 do
					lg.draw(
						style.tex.lineclear,
						style.clearquad[
							clamp(
								6 - floor(x + vars.timer/2 - 10),
								0,6
							)
						],
						style.board.x + x * block.size,
						style.board.y + i * block.size
					)
				end

				local text = ""
				if vars.lastclear == 1 then
					text = "SINGLE"
				elseif vars.lastclear == 2 then
					text = "DOUBLE"
				elseif vars.lastclear == 3 then
					text = "TRIPLE"
				elseif vars.lastclear == 4 then
					text = "TETRIS"
				end
				text = text:sub(0, math.floor(10 - vars.timer/2))
				drawfont(
					text,
					style.board.x + 8,
					style.board.y + i*block.size,
					-- we actually need to set the line clear bonus text
					-- to the color of the board palette. man what a mess.
					{
						palette[vars.level%(#palette+1)][2][1],
						palette[vars.level%(#palette+1)][2][2],
						palette[vars.level%(#palette+1)][2][3]
					}
				)
			end
		end
	elseif game.state == "gameover" then

	end

end,

}