local js_p1 = love.joystick.getJoysticks()[1]
local touch_isDown = {}
local stick_wasdown = {left = false, right = false, up = false, down = false}
local disabletouchafter = 2

-- load control configurations
local con = {}
for i,c in pairs(love.filesystem.getDirectoryItems("control")) do
	con[c] = {tex = {}}
	-- load configurations
	local conf = loadstring(love.filesystem.read("control/" .. c .. "/controls.lua"))()
	con[c].key, con[c].js, con[c].touch = conf.key, conf.js, conf.touch
	-- load icons
	for i,k in pairs(love.filesystem.getDirectoryItems("control/" .. c .. "/icon/")) do
		con[c].tex[k:sub(1,-5)] = lg.newImage("control/" .. c .. "/icon/" .. k)
	end
end

local function iskey(k, is_joystick)
	local c
	if is_joystick then
		c = con[state].js
	else
		c = con[state].key
	end

	for i in pairs(c) do
		for j = 1,#c[i] do
			if k == c[i][j] then
				return i
			end
		end
	end
	return k
end

function isdown(k)
	if js_p1 then
		local x,y = js_p1:getAxes()
		if x < 0 and k == "left"  then return true end
		if x > 0 and k == "right" then return true end
		if y < 0 and (k == "up"   or k == "hard") then return true end
		if y > 0 and (k == "down" or k == "soft")  then return true end
	end

	if con[state].key[k] then if love.keyboard.isDown(con[state].key[k]) then return true end end
	-- TODO bring this back
	--if love.keyboard.isDown(k) then return true end
	if con[state].js[k] and js_p1 then
		if js_p1:isGamepadDown(con[state].js[k]) then return true end
	end
	if touch_isDown[k] then return true end
	return false
end

function love.gamepadpressed(joystick, button)
	disabletouchafter = disabletouchafter - 1
	if disabletouchafter <= 0 then settings.touch = false end

	local k = iskey(button, true)
	main.input(k)
	if state == "game" and game and game.input then
		game.input(k)
	elseif state == "menu" then
		menu.input(k)
	end
end

function love.keypressed(key)
	disabletouchafter = disabletouchafter - 1
	if disabletouchafter <= 0 then settings.touch = false end

	local k = iskey(key)
	main.input(k)
	if state == "game" and game and game.input then
		game.input(k)
	elseif state == "menu" then
		menu.input(k)
	end
end

function touchdraw()
	lg.setColor(1,1,1,3/4)
	for i,k in pairs(con[state].touch) do
		if tonumber(i:sub(-1,-1)) ~= nil then i = i:sub(1,-2) end -- sets "hold2" to "hold"
		local x,y,w,h = k[1], k[2], k[3], k[4]
		local cx,cy = screen.w/8, screen.h/8
		local scale = min(cx,cy)/512
		-- draw buttons
		if touch_isDown[i] then
			lg.rectangle("fill", x*cx, y*cy, w*cx, h*cy, scale*64, scale*64, 4)
		end
		lg.rectangle("line", x*cx, y*cy, w*cx, h*cy, scale*64, scale*64, 4)
		-- draw button icons
		if con[state].tex[i] ~= nil then -- TODO this error handling might not be necessary in the final game
			lg.draw(
				con[state].tex[i],
				x*cx + cx*.5 + (w-1)*cx*.5,
				y*cy + cy*.5 + (h-1)*cy*.5,
				0,
				scale, scale,
				256, 256
			)
		end
	end
	lg.setColor(1,1,1,1)
end

function love.mousepressed(x,y,b)
	-- DEBUG
	if love.system.getOS() == "Linux" then love.touchpressed(0, x, y) end
end

function love.touchpressed(id, x, y, dx, dy, pressure)
	disabletouchafter = 2
	if not settings.touch then
		settings.touch = true
	else
		x = (x/lg.getWidth())*8
		y = (y/lg.getHeight())*8

		for i,j in pairs(con[state].touch) do
			if tonumber(i:sub(-1,-1)) ~= nil then i = i:sub(1,-2) end -- sets "hold2" to "hold"
			if x > j[1] and x < j[1] + j[3] and
			   y > j[2] and y < j[2] + j[4] then
				touch_isDown[i] = true
				main.input(i)
				if state == "game" and game and game.input then
					game.input(i)
				elseif state == "menu" then
					menu.input(i)
				end
			end
		end
	end
end

local function jstest(bool, dir)
	if bool then
		if not stick_wasdown[dir] then
			stick_wasdown[dir] = true
			love.keypressed(dir)
		end
	else
		stick_wasdown[dir] = false
	end
end

function controls_update()
	if js_p1 then
		local x,y = js_p1:getAxes()

		jstest(x < 0,"left")
		jstest(x > 0,"right")
		jstest(y < 0,"up")
		jstest(y > 0,"down")

	end

	if settings.touch then
		touch_isDown = {}
		for null, id in ipairs(love.touch.getTouches()) do
			local x,y = love.touch.getPosition(id)
			x = (x/lg.getWidth())*8
			y = (y/lg.getHeight())*8

			for i,j in pairs(con[state].touch) do
				if tonumber(i:sub(-1,-1)) ~= nil then i = i:sub(1,-2) end -- sets "hold2" to "hold"
				if x > j[1] and x < j[1] + j[3] and
				   y > j[2] and y < j[2] + j[4] then
				-- TODO not resetting this on switching states could be problematic
					touch_isDown[i] = true
				end
			end
		end
		--DEBUG
		if love.mouse.isDown(1) then
			local x,y = (love.mouse.getX()/lg.getWidth())*8,(love.mouse.getY()/lg.getHeight())*8
			for i,j in pairs(con[state].touch) do
				if x > j[1] and x < j[1] + j[3] and
				   y > j[2] and y < j[2] + j[4] then
					touch_isDown[i] = true
				end
			end
		end
	end
end