version = "63ab334b078c526676d4901813ea101c528fe52f"
--version = "v0.0.0 Unclean"

--~ debug = true
--~ if debug then printd = print
--~ else printd = function() end
--~ end

warning = require("warning")

lg = love.graphics

function newgame(type)

	-- load style's textures
	-- TODO only do this on new style
	local reloadtex = function()
		local tex = {}
		get = function(folder,obj)
			local fs = love.filesystem.getDirectoryItems(folder)
			for i,k in pairs(fs) do

				if love.filesystem.getInfo(folder .. "/" .. k).type == "directory" then
					obj[k] = {}
					obj[k] = get(folder .. "/" .. k, obj[k])
				else
					obj[k:sub(1,-5)] = lg.newImage(folder .. "/" .. k)
				end
			end
			return obj
		end
		style.tex = get("games/" .. settings.game .. "/style/" .. settings.style .. "/tex", {})
	end

	if type == "style" then
		local oldstyle = style
		love.filesystem.load("games/" .. settings.game .. "/style/" .. settings.style .. "/script.lua")()
		if oldstyle.board.w ~= style.board.w
		or oldstyle.board.h ~= style.board.h
		or game.state == "gameover" then
			game.reload.board()
			game.state = "playing"
		end
		game.reload.sfx()
		reloadtex()
		game.reload.style()
	elseif type == "board" then
		game.reload.board()
		updatemusic()
		game.reload.style() -- reload palette on new game, mostly
	else
		--TODO can we call newgame("style")?
		if not exists("games/" .. settings.game .. "/style/" .. settings.style) then
			settings.style = love.filesystem.getDirectoryItems("games/" .. settings.game .. "/style/")[1]
		end
		love.filesystem.load("games/" .. settings.game .. "/style/" .. settings.style .. "/script.lua")()
		love.filesystem.load("games/" .. settings.game .. "/game.lua")()

		if not settings.conf then
			settings.conf = shallowcopy(style.conf)
			savegame()
		end

		game.reload.gfx()
		game.reload.sfx()
		game.reload.board()
		reloadtex()
		game.reload.style()
		updatemusic()
	end

	collectgarbage()

	love.resize()
end

function love.load()
	exists = function(str)
		return love.filesystem.getInfo(str) ~= nil
	end
	lg.setDefaultFilter("nearest", "nearest")

	fontscale = 2
	font = lg.newFont("gulim.ttf", 30*fontscale)
	font:setFilter("linear","linear")
	lg.setFont(font)

	require "err"
	require "maths"
	require "func"
	require "input"
	require "sound"
	require "drawhelpers"
	require "serial"
	require "settings"
	require "menu"

	shader = {}
	for i,k in pairs(love.filesystem.getDirectoryItems("/shaders")) do
		shader[k:sub(1,-6)] = lg.newShader("/shaders/" .. k)
	end

	state = "menu"

	bg = {}
	frametime = 0

	redraw = {
		game = true,
		main = true,
	}

	music = {
		isfast = false,
		fastspeed = 0, -- how much faster the fast music is compared to normal
		-- source = automatically loaded
		-- fast = automatically loaded
	}
	
	love.audio.setEffect("reverb1", {type = "reverb"})

	if settings.gfx.fullscreen then
		love.window.setMode(lg.getWidth(), lg.getHeight(), {vsync=not settings.gfx.lowlatency, resizable=true, fullscreen=settings.gfx.fullscreen, usedpiscale=false})
	else
		love.window.setMode(settings.window.w, settings.window.h, {vsync=not settings.gfx.lowlatency, resizable=true, fullscreen=settings.gfx.fullscreen, usedpiscale=false})
	end

	love.resize()

	newgame()
	menu.init()
end

function love.update(dt)
	controls_update()
	if bg.update then bg.update(dt) end
	warning.update(dt)

	if settings.gfx.lowlatency then
		frametime = frametime - dt
		if dt < 1/60 then
			-- gives your CPU a slight break
			love.timer.sleep(1/120)
		end
		dt = 1/60
		redraw.main = true
		-- if more than 1/60 sec passed, we need to do multiple frames
		-- we might not need to do any at all though
		while frametime < 0 do
			frametime = frametime + 1/60
			if state == "game" and game then
				game.update(dt)
				redraw.game = true
			end
		end
	else
		if state == "game" and game then
			game.update(dt)
		end
	end
end

function love.draw()
	if redraw.main or not settings.gfx.lowlatency then
		redraw.main = false

		-- menu bg
		if settings.gamebg and state == "game" then bg.draw() end

		-- draw game
		local c, w, h = game.draw(redraw.game or not settings.gfx.lowlatency)
		local rw,rh = c:getWidth(), c:getHeight() -- real width/height
		lg.push()
		local x,y,scale = centerscreen(w,h, settings.gfx.intscale)
		lg.scale(w/rw, h/rh)

		if settings.gfx.crt then
			-- TODO don't do this every frame
			shader.scanline:send("screen", {rw,rh})
			shader.bloom:send("screen", {rw,rh})
			c:setFilter("linear","linear")

			lg.setCanvas(canvas.shader)
			lg.setShader(shader.scanline)
			lg.clear()
			lg.draw(c)
			lg.setCanvas()
			lg.pop()

			lg.setCanvas()
			lg.setShader(shader.bloom)
			lg.draw(canvas.shader)
			lg.setShader()
		else
			c:setFilter("nearest","nearest")
			if settings.gfx.pixelshader and not (settings.gfx.intscale and not settings.gfx.aspectcorrect) then
				lg.setShader(shader.pixellate)
				-- TODO make this not every frame
				local range = {
					abs(rw / (rw*scale * rw))/ 2.0 * 0.999,
					abs(rh / (rh*scale * rh))/ 2.0 * 0.999
				}
				shader.pixellate:send("texelSize", {1/rw, 1/rh})
				shader.pixellate:send("range", range)
			end
			lg.draw(c)
			lg.pop()
			lg.setShader()
		end

		-- menu
		if state == "menu" then
			lg.setColor(0,0,0,3/4)
			lg.rectangle("fill", 0, 0, screen.w, screen.h)
			lg.setColor(1,1,1)
			bg.draw()
			menu.draw()
		end
		if settings.gfx.showfps or love.timer.getFPS() < 60 then
			if love.timer.getFPS() < 60 then lg.setColor(1,0.25,0) end
			lg.print(
				love.timer.getFPS() .. " FPS\n",
				0,0,0,min(screen.w,screen.h)/(512*fontscale)
			)
			lg.setColor(1,1,1)
		end
		if settings.touch then
			touchdraw()
		end
		lg.setCanvas()
	end

	warning.draw()
end

function love.resize(w,h)
	w = w or lg.getWidth()
	h = h or lg.getHeight()
	if not settings.gfx.fullscreen then
		settings.window = {
			w = w,
			h = h,
		}
	end
	screen = {w = w, h = h}
	menu.resize(w,h)
	canvas = {
		game = lg.newCanvas(w,h),
		shader = lg.newCanvas(w,h),
	}
	redraw.game = true
end

function love.quit()
	savegame()
end

main = {}
function main.input(k)
	if k == "fullscreen" then
		-- TODO maybe make togglefullscreen()?
		settings.gfx.fullscreen = not settings.gfx.fullscreen
		if settings.gfx.fullscreen then
			love.window.setMode(lg.getWidth(), lg.getHeight(), {vsync=not settings.gfx.lowlatency, resizable=true, fullscreen=settings.gfx.fullscreen, usedpiscale=false})
		else
			love.window.setMode(settings.window.w, settings.window.h, {vsync=not settings.gfx.lowlatency, resizable=true, fullscreen=settings.gfx.fullscreen, usedpiscale=false})
		end
		newgame("style") -- re-runs style.init TODO find out why this is necessary
		menu.init() -- mostly to update the menu option for full screen
	end
end