// Pixellate Shader
// Copyright (c) 2011, 2012 Fes
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
extern vec2 texelSize;
extern vec2 range;

vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
{
	float left   = uv.x - range.x;
	float top    = uv.y + range.y;
	float right  = uv.x + range.x;
	float bottom = uv.y - range.y;

	vec4 topLeftColor;
	vec4 bottomRightColor;
	vec4 bottomLeftColor;
	vec4 topRightColor;

	topLeftColor     = Texel(tex, (floor(vec2(left, top)     / texelSize) + 0.5) * texelSize);
	bottomRightColor = Texel(tex, (floor(vec2(right, bottom) / texelSize) + 0.5) * texelSize);
	bottomLeftColor  = Texel(tex, (floor(vec2(left, bottom)  / texelSize) + 0.5) * texelSize);
	topRightColor    = Texel(tex, (floor(vec2(right, top)    / texelSize) + 0.5) * texelSize);

	vec2 border = clamp(floor(uv / texelSize + 0.5) * texelSize, vec2(left, bottom), vec2(right, top));

	float totalArea = 4.0 * range.x * range.y;

	vec4 averageColor;
	averageColor  = ((border.x - left)  * (top - border.y)    / totalArea) * topLeftColor;
	averageColor += ((right - border.x) * (border.y - bottom) / totalArea) * bottomRightColor;
	averageColor += ((border.x - left)  * (border.y - bottom) / totalArea) * bottomLeftColor;
	averageColor += ((right - border.x) * (top - border.y)    / totalArea) * topRightColor;

	return averageColor;
}